#ifndef FmStringDefs_h
#define FmStringDefs_h

/*
 *  Enhancements to the X-File Manager XFM-1.3.2 
 *  --------------------------------------------
 
 *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>

 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

/*
 *  String definitions for the FileList, IconFileList and TextFileList widgets.
 */

extern char FmResStrings[];
extern char FmClStrings[];
extern char FmTypeStrings[];

/* THIS FILE WAS AUTOMATICALLY GENERATED BY SCAN_STRINGS */
/************************* DO NOT EDIT *******************/

extern char	FmResStrings[];
extern char	FmClStrings[];
extern char	FmTypeStrings[];

#ifndef XtNfiles
#define XtNfiles ((char *)(&FmResStrings[0]))
#endif
#ifndef XtCFiles
#define XtCFiles ((char *)(&FmClStrings[0]))
#endif
#ifndef XtNnFiles
#define XtNnFiles ((char *)(&FmResStrings[6]))
#endif
#ifndef XtCNFiles
#define XtCNFiles ((char *)(&FmClStrings[6]))
#endif
#ifndef XtNentrySep
#define XtNentrySep ((char *)(&FmResStrings[13]))
#endif
#ifndef XtCEntrySep
#define XtCEntrySep ((char *)(&FmClStrings[13]))
#endif
#ifndef XtNleftSep
#define XtNleftSep ((char *)(&FmResStrings[22]))
#endif
#ifndef XtCLeftSep
#define XtCLeftSep ((char *)(&FmClStrings[22]))
#endif
#ifndef XtNtopSep
#define XtNtopSep ((char *)(&FmResStrings[30]))
#endif
#ifndef XtCTopSep
#define XtCTopSep ((char *)(&FmClStrings[30]))
#endif
#ifndef XtNtabSep
#define XtNtabSep ((char *)(&FmResStrings[37]))
#endif
#ifndef XtCTabSep
#define XtCTabSep ((char *)(&FmClStrings[37]))
#endif
#ifndef XtNlabelSep
#define XtNlabelSep ((char *)(&FmResStrings[44]))
#endif
#ifndef XtCLabelSep
#define XtCLabelSep ((char *)(&FmClStrings[44]))
#endif
#ifndef XtNhighlightColor
#define XtNhighlightColor ((char *)(&FmResStrings[53]))
#endif
#ifndef XtCHighlightColor
#define XtCHighlightColor ((char *)(&FmClStrings[53]))
#endif
#ifndef XtNshowInode
#define XtNshowInode ((char *)(&FmResStrings[68]))
#endif
#ifndef XtCShowInode
#define XtCShowInode ((char *)(&FmClStrings[68]))
#endif
#ifndef XtNshowType
#define XtNshowType ((char *)(&FmResStrings[78]))
#endif
#ifndef XtCShowType
#define XtCShowType ((char *)(&FmClStrings[78]))
#endif
#ifndef XtNshowLinks
#define XtNshowLinks ((char *)(&FmResStrings[87]))
#endif
#ifndef XtCShowLinks
#define XtCShowLinks ((char *)(&FmClStrings[87]))
#endif
#ifndef XtNshowPermissions
#define XtNshowPermissions ((char *)(&FmResStrings[97]))
#endif
#ifndef XtCShowPermissions
#define XtCShowPermissions ((char *)(&FmClStrings[97]))
#endif
#ifndef XtNshowOwner
#define XtNshowOwner ((char *)(&FmResStrings[113]))
#endif
#ifndef XtCShowOwner
#define XtCShowOwner ((char *)(&FmClStrings[113]))
#endif
#ifndef XtNshowGroup
#define XtNshowGroup ((char *)(&FmResStrings[123]))
#endif
#ifndef XtCShowGroup
#define XtCShowGroup ((char *)(&FmClStrings[123]))
#endif
#ifndef XtNshowLength
#define XtNshowLength ((char *)(&FmResStrings[133]))
#endif
#ifndef XtCShowLength
#define XtCShowLength ((char *)(&FmClStrings[133]))
#endif
#ifndef XtNshowDate
#define XtNshowDate ((char *)(&FmResStrings[144]))
#endif
#ifndef XtCShowDate
#define XtCShowDate ((char *)(&FmClStrings[144]))
#endif
#ifndef XtNnHoriz
#define XtNnHoriz ((char *)(&FmResStrings[153]))
#endif
#ifndef XtCNHoriz
#define XtCNHoriz ((char *)(&FmClStrings[153]))
#endif
#ifndef XtNminIconWidth
#define XtNminIconWidth ((char *)(&FmResStrings[160]))
#endif
#ifndef XtCMinIconWidth
#define XtCMinIconWidth ((char *)(&FmClStrings[160]))
#endif
#ifndef XtNminIconHeight
#define XtNminIconHeight ((char *)(&FmResStrings[173]))
#endif
#ifndef XtCMinIconHeight
#define XtCMinIconHeight ((char *)(&FmClStrings[173]))
#endif
#ifndef XtNuserData
#define XtNuserData ((char *)(&FmResStrings[187]))
#endif
#ifndef XtCUserData
#define XtCUserData ((char *)(&FmClStrings[187]))
#endif
#ifndef XtNdragTimeout
#define XtNdragTimeout ((char *)(&FmResStrings[196]))
#endif
#ifndef XtCDragTimeout
#define XtCDragTimeout ((char *)(&FmClStrings[196]))
#endif


#ifndef XtRFileList
#define XtRFileList ((char *)(&FmTypeStrings[0]))
#endif

#endif
