#ifndef _XtHelper_h
#define _XtHelper_h
/*
 *  Enhancements to the X-File Manager XFM-1.3.2 (Xt Helper functions)
 *  ------------------------------------------------------------------
 
 *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>

 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

/* Xt Helper Functions */

#include <X11/Intrinsic.h>

/* Xt doesn't export a function/macro to find out whether a
 * widget is being destroyed :-(
 */

Boolean
XtIsBeingDestroyed(Widget w);

#endif
