/* THIS FILE WAS AUTOMATICALLY GENERATED BY SCAN_STRINGS */
/************************* DO NOT EDIT *******************/
#ifndef _FocusForm_h
#define _FocusForm_h
/*
 *  $id$
 *
 *  The FocusForm Widget
 *  --------------------
 
 *  Copyright (C) 1997  by Till Straumann   <strauman@sun6hft.ee.tu-berlin.de>

 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Library Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Library Public License for more details.

 *  You should have received a copy of the GNU General Library Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Log: FocusForm.h,v $
 *  Revision 1.1.1.1.4.1  2000/02/23 16:21:52  till
 *   - merged albert graef's changes from p3 to p4
 *
 *  Revision 1.1  1999/10/18 22:37:57  cvs
 *   - Imakefile supports building a (static) library now.
 *   - all (resource etc.) strings go to strDefs.c if SEPARATE_STRINGS is defined.
 *
 *  Revision 1.2  1998/04/19 11:00:25  till
 *  started cvs logging
 *

 */

/* This is a subclass of the athena 'Form' widget */
/* Copyright (c) 1987, 1988, 1994  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
*/

#include <X11/Xaw/Form.h>

/***********************************************************************
 *
 * FocusForm Widget (Subclass of Form)
 *
 * The FocusForm widget provides additional resources in order to control
 * focus traversal.
 *
 ***********************************************************************/

/* Parameters:

 Name		     Class		RepType		Default Value	Description
 ----		     -----		-------		-------------	-----------
 focusInitChild	     FocusInitChild	String		NULL		(qualified) name of
									a child to whom to give
									the focus after realizing
									the widget tree. The name
									is evaluated by
									XtName(top_level_widget,focusInitChild)

*/

/* Constraint parameters:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 focusInterest	     FocusInterest	Boolean		False		Child is willing to
									accept the focus.
 sensitive	     Sensitive		Boolean		True		Overrides the core
									resource. This is used
									to redirect the focus
									upon setting to insensitive.
									(handled transparently)

*/

/* 
 * Actions and Translations
 *
 * The following actions are provided:
 *
 *   FocusTraverse([f/b]) Give the focus to the next child of the FocusForm, which is willing
 *			to accept it (sensitive && focusInterest && managed). If this action
 *			is invoked by a non FocusForm widget, the closest FocusForm-ancestor
 *			is retrieved and performs the action. (Optional parameter indicates
 *			the direction: forward / backward)
 *
 *   FocusSet( [qualified widget name] ) Set the focus to the widget invoking this action. If
 *			the optional, qualified name resolves to a valid (sensitive etc.)
 *			widget relative to the toplevel, focus is transferred there.
 *
 * The default translations for the FocusForm are
 *
 * Shift<Key>Tab:  FocusTraverse(b)\n\
 *	<Key>Tab:  FocusTraverse()\n\
 *	<BtnUp> :  FocusSet()\n\
 *	<Enter> :  FocusSet()
 *
 */


/* THIS FILE WAS AUTOMATICALLY GENERATED BY SCAN_STRINGS */
/************************* DO NOT EDIT *******************/

#ifdef __STDC__
extern const char	FocusFormResStrings[];
extern const char	FocusFormClStrings[];
extern const char	FocusFormTypeStrings[];
#else
extern char	FocusFormResStrings[];
extern char	FocusFormClStrings[];
extern char	FocusFormTypeStrings[];
#endif

#ifndef XtNfocusInterest
#define XtNfocusInterest ((char *)(&FocusFormResStrings[0]))
#endif
#ifndef XtCFocusInterest
#define XtCFocusInterest ((char *)(&FocusFormClStrings[0]))
#endif
#ifndef XtNfocusInitChild
#define XtNfocusInitChild ((char *)(&FocusFormResStrings[14]))
#endif
#ifndef XtCFocusInitChild
#define XtCFocusInitChild ((char *)(&FocusFormClStrings[14]))
#endif

typedef struct _FocusFormClassRec	*FocusFormWidgetClass;
typedef struct _FocusFormRec		*FocusFormWidget;

extern WidgetClass focusFormWidgetClass;

#endif /* _FocusForm_h */
