/*-----------------------------------------------------------------------------
  FmComms.h

  (c) Simon Marlow 1992
  (c) Albert Graef 1994

  atom definitions for client messages
-----------------------------------------------------------------------------*/

#ifndef FMCOMMS_H
#define FMCOMMS_H

#define XFM_OPEN_WINDOW   "XFM_OPEN_WINDOW"
#define XFM_UPDATE_WINDOW "XFM_UPDATE_WINDOW"
#define WM_DELETE_WINDOW  "WM_DELETE_WINDOW"
#define WM_PROTOCOLS      "WM_PROTOCOLS"
#define WM_SAVE_YOURSELF  "WM_SAVE_YOURSELF"
#define KWM_SAVE_YOURSELF "KWM_SAVE_YOURSELF"

#endif
